import pandas as pd
from scipy.stats import linregress
from rltrade import config
from rltrade.data import DiffrencedFeatureEngineer,load_csv

csv_path = 'testdata/meta.csv'

tech_indicators = config.STOCK_INDICATORS_LIST # indicators from stockstats
additional_indicators = config.ADDITIONAL_DIFF_STOCK_INDICATORS
all_indicators = tech_indicators + additional_indicators

df = load_csv(csv_path) 
fe = DiffrencedFeatureEngineer(additional_indicators=additional_indicators,
                    stock_indicator_list=tech_indicators) 

df = fe.create_data(df)
df['daily_return'] = df.groupby('tic')['close'].pct_change(1)
df = df.dropna()

r2_list = []
for indicator in tech_indicators+additional_indicators:
    r2_score = linregress(df['daily_return'],df[indicator])[2]
    r2_list.append([indicator,r2_score])

df_score = pd.DataFrame(r2_list,columns=['feature','r2_score'])
df_score = df_score.sort_values(by=['r2_score'],ascending=False)
print(df_score)
df_score.to_csv('r2featureimportance.csv',index=False)


import asyncio
import pandas as pd
from scipy.stats import norm

from rltrade import config
from metaapi_cloud_sdk import MetaApi
from rltrade.data import OandaDownloader,FeatureEngineer
from rltrade.models import SmartTradeAgent


"""
time_frame - last date available
4h  - '2021-04-23'
1h  - '2021-10-19'
30m - '2021-11-18'
15m - '2021-12-03'
5m -  '2021-12-08'
"""

time_frame = "1d" # available 5m, 15m, 30m, 1h, 4h
train_period = ('2018-01-07','2021-12-15') #for training the model
test_period = ('2021-12-15','2021-12-17') 
start_time = "00:00:00"
end_time = "23:55:00" 
path = 'models/daytrades/forex'

token = 'eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.eyJfaWQiOiIzYTI0YzMwYzFkNDRmZGFmZGI3NmVmYTUxZmQ2MDJmMCIsInBlcm1pc3Npb25zIjpbXSwidG9rZW5JZCI6IjIwMjEwMjEzIiwiaWF0IjoxNjM5NDkwMzEzLCJyZWFsVXNlcklkIjoiM2EyNGMzMGMxZDQ0ZmRhZmRiNzZlZmE1MWZkNjAyZjAifQ.DhA6Q-AlWNssi8ScOaWy2Bxo4Hebgw94BDE6PiT9J-TvsuF7OqvdYBQ1IWYEMtsYsg3Ij8p8Wpvn8ZdZeHRkR3vLcQnMH-GZj3DyYkeqovQKk6U3uOobV-GS3meJPZYfw2zItuTDBWxuHDZsVW1ZvF4sItBDmsWIe2svF0NmKE1nu-ephcYVzYo9grr93de_h-QwlP-yeZFeGEqrz3-q5gYWcARJsIR1BX63zePuDHkUK9k5W9Rm28WdB87MHEyMSWhcAZDf8si5MwsPYC3wpzNtzGqORF3UY-w5EmolCtSPMBqM7AI0LKc1n8GPS3ZhnvHkfGhWEdb5gKlCWwshk30tICN24C1bZG06zfs450oLm8ih9ls5oyshcg_xwawNvsA305D7Siz0Pzqr1xnUA8zMz8cVUFZtjdBWCfot05_ziVO0x_mApVyAVC2OA-Sh61RtkwNNpg4bTCzK30OpdiS9GO0HLgnepnuwWOO0T9DTzTAJUxyJcXOzcWcXGdMTWaGAp5ranytU97k8GxDHa5jOS_WvphL24C8QA6of0pYZwHM3Ul5Aw351H1SbJLIqs2AChDoUnpJb9OZb-27ESLZgM1mhU6rwzt8lRRbxUHaXSv5QpM29nPm3k5KrFSv-UTXiX6oU9c1nNh3qLb6FKV_B1CQarcvyx66iUg-1DcU'
login = '2132970485'
password = 'Marines791!'
account_id = 'ddc8fb93-e0f5-4ce8-b5d5-8290d23fc143'
server_name = 'CBTLimited-Trading'
broker_srv_file = '/home/maunish/Upwork Projects/rl-trade/examples/CBTLimited-Trading.srv'
domain = 'agiliumtrade.agiliumtrade.ai'

ticker_list = ['AUDUSD','EURUSD','NZDUSD','USDCAD','USDCHF','GBPUSD']
sec_types = ['-','-','-','-','-','-']
exchanges = ['-','-','-','-','-','-']


tech_indicators = config.STOCK_INDICATORS_LIST # indicators from stockstats
additional_indicators = config.ADDITIONAL_STOCK_INDICATORS

env_kwargs = {
    "initial_amount": 50_000, #this does not matter as we are making decision for lots and not money.
    "ticker_col_name":"tic",
    "mode":'min',
    "stop_loss":0.015,
    "filter_threshold":1, #between 0.1 to 1, select percentage of top stocks 0.3 means 30% of top stocks
    "target_metrics":['asset'], #asset, cagr, sortino, calamar, skew and kurtosis are available options.
    "transaction_cost":0, #transaction cost per order
    "tech_indicator_list":tech_indicators + additional_indicators, 
    "reward_scaling": 1}

PPO_PARAMS = {'ent_coef':0.005,
            'learning_rate':0.01,
            'batch_size':522}

def get_buckets(df,bucketSize):
    volumeBuckets = pd.DataFrame(columns=['buy','sell','date'])
    count = 0
    BV = 0
    SV = 0
    for index,row in df.iterrows():
        newVolume = row['volume']
        z = row['z']
        if bucketSize < count + newVolume:
            BV = BV + (bucketSize-count)*z
            SV = SV + (bucketSize-count)*(1-z)
            volumeBuckets = volumeBuckets.append({'buy':BV, 'sell':SV, 'date':index},ignore_index=True)
            count = newVolume-(bucketSize-count)
            if int(count/bucketSize) > 0:
                for i in range(0,int(count/bucketSize)):
                    BV = (bucketSize)*z
                    SV = (bucketSize)*(1-z)
                    volumeBuckets = volumeBuckets.append({'buy':BV, 'sell':SV, 'date':index},ignore_index=True)

            count = count%bucketSize
            BV = (count)*z
            SV = (count)*(1-z)
        else:
            BV = BV + (newVolume)*z
            SV = SV + (newVolume)*(1-z)
            count = count + newVolume

    volumeBuckets = volumeBuckets.drop_duplicates('date',keep='last')
    volumeBuckets = volumeBuckets.set_index('date')
    return volumeBuckets

def calc_vpin(data, bucketSize,window):
    
    volume = (data['volume'])
    trades = (data['close'])
    
    trades_1min = trades.diff(1).dropna()
    volume_1min = volume.dropna()
    sigma = trades_1min.std()
    z = trades_1min.apply(lambda x: norm.cdf(x/sigma))
    df = pd.DataFrame({'z': z, 'volume': volume_1min}).dropna()
    
    volumeBuckets=get_buckets(df,bucketSize)
    volumeBuckets['VPIN'] = abs(volumeBuckets['buy']-volumeBuckets['sell']).rolling(window).mean()/bucketSize
    volumeBuckets['CDF'] = volumeBuckets['VPIN'].rank(pct=True)
    return volumeBuckets

async def train_model():
    api = MetaApi(token,{'domain':domain})

    try:
        account = await api.metatrader_account_api.get_account(account_id)

        if account.state != 'DEPLOYED':
            await account.deploy()
        else:
            print('Account already deployed')
        if account.connection_status != 'CONNECTED':
            await account.wait_connected()
    
        oa = OandaDownloader(
            account = account,
            time_frame=time_frame,
            start_date=train_period[0],
            end_date=test_period[1],
            start_time=start_time,
            end_time=end_time,
            ticker_list=ticker_list)

        df = await oa.fetch_data()

        df = df.sort_values(by=['date','tic'])
        df.to_csv('testdata/dft.csv',index=False)

        df = pd.read_csv('testdata/dft.csv')
        df['date'] = pd.to_datetime(df['date'])

        # agent = SmartTradeAgent("ppo",
        #                     df=df,
        #                     account=account,
        #                     time_frame=time_frame,
        #                     ticker_list=ticker_list,
        #                     sec_types = sec_types,
        #                     exchanges=exchanges,
        #                     ticker_col_name="tic",
        #                     tech_indicators=tech_indicators,
        #                     additional_indicators=additional_indicators,
        #                     train_period=train_period,
        #                     test_period=test_period,
        #                     start_time=start_time,
        #                     end_time=end_time,
        #                     env_kwargs=env_kwargs,
        #                     model_kwargs=PPO_PARAMS,
        #                     tb_log_name='ppo',
        #                     mode='min', # daily or min
        #                     epochs=2)

        # agent.train_model()
        # agent.save_model(path) #save the model for trading

        # df_daily_return,df_actions = agent.make_prediction() #testing model on testing period

    except Exception as err:
        print(api.format_error(err))


# asyncio.run(train_model())
df = pd.read_csv('testdata/dft.csv').query("tic == 'EURUSD'")
df['date'] = pd.to_datetime(df['date'])
df = df.head(50)
# df = df.set_index('date').head(50)

fe = FeatureEngineer(additional_indicators=additional_indicators,
                    stock_indicator_list=tech_indicators,
                    cov_matrix=True) 

temp = fe.add_vpin(df,10,'vpin_10')

print(temp)




from rltrade import config
from rltrade.data import load_csv
from rltrade.backtests import get_metrics
from rltrade.models import SmartDayTradeAgent3


"""
time_frame - last date available
4h  - '2017-12-01'
1h  - '2020-06-19'
30m - '2021-11-18'
15m - '2021-12-03'
5m -  '2021-10-01'
"""

time_frame = "5m" # available 5m, 15m, 30m, 1h, 4h
train_period = ('2021-10-01','2021-12-01') #for training the model
test_period = ('2021-12-01','2021-12-20') 
start_time = "00:00:00"
end_time = "23:55:00" 

csv_path = 'testdata/meta1d.csv'
path = 'models/daytrades/forex'

ticker_list = ['AUDUSD','EURUSD','NZDUSD','USDCAD','USDCHF','GBPUSD']
# ticker_list = ['USDCAD','USDCHF']
sec_types = ['-'] * len(ticker_list)
exchanges = ['-'] * len(ticker_list)
trades_per_stock = [200] * len(ticker_list) # max number per day is 24h/time_frame

# tech_indicators = config.STOCK_INDICATORS_LIST # indicators from stockstats
# additional_indicators = config.ADDITIONAL_DAYTRADE_INDICATORS

tech_indicators = [
    "boll_ub",
    "boll_lb",
    "close_30_sma",
    "close_60_sma",
    "open_2_sma",
    "boll",
	"close_2_tema",
	"tema",
]

additional_indicators = [
    'sortino_diff_100',
    'sortino_diff_1000',
    'max_value_volume_diff_1000',
    'sortino_500',
    'vix_fix_1000',
    'max_value_price_10',
    'max_value_price_50',
    'max_value_price_100',
    'max_value_price_500',
    'max_value_price_1000',
    'min_value_price_10',
    'min_value_price_50',
    'min_value_price_100',
    'min_value_price_1000',
    'max_value_volume_50',
    'max_value_volume_500',
    'min_value_volume_500',
    'min_value_volume_1000',
]


env_kwargs = {
    "initial_amount": 17_500, #this does not matter as we are making decision for lots and not money.
    "ticker_col_name":"tic",
    "stop_loss":0.015,
    "trades_per_stock":trades_per_stock,
    "filter_threshold":1, #between 0.1 to 1, select percentage of top stocks 0.3 means 30% of top stocks
    "target_metrics":['asset'], #asset, cagr, sortino, calamar, skew and kurtosis are available options.
    "transaction_cost":0, #transaction cost per order
    "tech_indicator_list":tech_indicators + additional_indicators, 
    "reward_scaling": 1}

PPO_PARAMS = {'ent_coef':0.005,
            'learning_rate':0.01,
            'batch_size':522}

def train_model():

    df = load_csv(csv_path)

    agent = SmartDayTradeAgent3("ppo",
                        df=df,
                        account=None,
                        time_frame=time_frame,
                        ticker_list=ticker_list,
                        sec_types = sec_types,
                        exchanges=exchanges,
                        ticker_col_name="tic",
                        tech_indicators=tech_indicators,
                        additional_indicators=additional_indicators,
                        train_period=train_period,
                        test_period=test_period,
                        start_time=start_time,
                        end_time=end_time,
                        env_kwargs=env_kwargs,
                        model_kwargs=PPO_PARAMS,
                        tb_log_name='ppo',
                        epochs=10)

    agent.train_model()
    agent.save_model(path) #save the model for trading

    df_daily_return,df_actions = agent.make_prediction() #testing model on testing period
    get_metrics(path)

train_model()


import os
import pandas as pd
import plotly.graph_objects as go
from scipy.stats.stats import pearsonr
from scipy.stats import ttest_ind,linregress


def get_metrics(path,live=False):
    path = os.path.join(os.getcwd(),path)
    train_df = pd.read_csv(path+'/train_df.csv')
    train_df['date'] = pd.to_datetime(train_df['date'])
    train_daily_return = train_df.groupby('date')['return with cost'].sum()
    train_total_daily_return = train_daily_return.sum()

    test_df = pd.read_csv(path+'/test_df.csv')
    test_df['date'] = pd.to_datetime(test_df['date'])
    test_daily_return = test_df.groupby('date')['return with cost'].sum()
    test_total_daily_return = test_daily_return.sum()

    train_sortino = (252 **0.5) * train_daily_return.mean()/train_daily_return[train_daily_return<=0].std()
    test_sortino = (252 ** 0.5) * test_daily_return.mean()/test_daily_return[test_daily_return<=0].std()

    train_avg_return= train_daily_return.mean()
    train_max_drawdown = train_daily_return.diff(1).min()
    train_calamar = train_avg_return/ train_max_drawdown

    test_avg_return= test_daily_return.mean()
    test_max_drawdown = test_daily_return.diff(1).min()
    test_calamar = test_avg_return/ test_max_drawdown

    train_test_p_value = ttest_ind(train_daily_return,test_daily_return)[1]
    
    if len(train_daily_return) >= len(test_daily_return):
        train_test_corr = pearsonr(train_daily_return.tail(len(test_daily_return)).values,test_daily_return.values)[0]
        train_test_rvalue = linregress(train_daily_return.tail(len(test_daily_return)).values,test_daily_return.values)[2]
    else:
        train_test_corr = pearsonr(test_daily_return.tail(len(train_daily_return)).values,train_daily_return.values)[0]
        train_test_rvalue = linregress(test_daily_return.tail(len(train_daily_return)).values,train_daily_return.values)[2]

    if os.path.exists(path+'/live_df.csv') and live:
        live_df = pd.read_csv(path+'/live_df.csv')
        live_df = live_df.drop_duplicates('date')
        live_daily_return = live_df.groupby('date')['return with cost'].sum()
        live_total_daily_return = live_daily_return.sum()
        live_sortino = (252 ** 0.5) * live_daily_return.mean() / live_daily_return[live_daily_return<=0].std()
        live_calamar = live_daily_return.mean()/live_daily_return.diff(1).min()
        train_live_p_value = ttest_ind(train_daily_return,live_daily_return)[1]
        test_live_p_value = ttest_ind(test_daily_return,live_daily_return)[1]
        if len(train_daily_return) >= len(live_daily_return):
            train_live_corr = pearsonr(train_daily_return.tail(len(live_daily_return)).values,live_daily_return.values)[0]
            train_live_rvalue = linregress(train_daily_return.tail(len(live_daily_return)).values,live_daily_return.values)[2]
        else:
            train_live_corr = pearsonr(live_daily_return.tail(len(train_daily_return)).values,train_daily_return.values)[0]
            train_live_rvalue = linregress(live_daily_return.tail(len(train_daily_return)).values,train_daily_return.values)[2]

        if len(test_daily_return)>= len(live_daily_return):
            test_live_corr = pearsonr(test_daily_return.tail(len(live_daily_return)).values,live_daily_return.values)[0]
            test_live_rvalue = linregress(test_daily_return.tail(len(live_daily_return)).values,live_daily_return.values)[2]
        else:
            test_live_corr = pearsonr(live_daily_return.tail(len(test_daily_return)).values,test_daily_return.values)[0]
            test_live_rvalue = linregress(live_daily_return.tail(len(test_daily_return)).values,test_daily_return.values)[2]

        metrics_df = pd.DataFrame([['sortino',train_sortino,test_sortino,live_sortino],
                                ['calamar',train_calamar,test_calamar,live_calamar],
                                ['total return',train_total_daily_return,test_total_daily_return,live_total_daily_return],
                                ['train live corr',train_live_corr,'-',train_live_corr],
                                ['test live corr','-',test_live_corr,test_live_corr],
                                ['train live p-value',train_live_p_value,'-',train_live_p_value],
                                ['test live p-value','-',test_live_p_value,test_live_p_value],
                                ['train live r2-value',train_live_rvalue**2,'-',train_live_rvalue**2],
                                ['test live r2-value','-',test_live_rvalue**2,test_live_rvalue**2]])

        metrics_df.columns = ['metrics','train','test','live']
    
    else:
        metrics_df = pd.DataFrame([['sortino',train_sortino,test_sortino],
                                    ['calamar',train_calamar,test_calamar],
                                     ['total return',train_total_daily_return,test_total_daily_return],
                                     ['corr',train_test_corr,train_test_corr],
                                     ['p-value',train_test_p_value,train_test_p_value],
                                     ['r2-value',train_test_rvalue**2,train_test_rvalue**2]])

        metrics_df.columns = ['metrics','train','test']

    print(metrics_df)

def get_symbol_df(path,symbol):
    path = os.path.join(os.getcwd(),path)
    df_train = pd.read_csv(path+'/train_df.csv')
    df_test = pd.read_csv(path+'/test_df.csv')
    df = pd.concat([df_train,df_test])
    df.drop_duplicates(['date','tic'],inplace=True)
    df = df[df['tic'] == symbol]
    df.reset_index(inplace=True)
    df["date"] = pd.to_datetime(df["date"])
    return df

def plot_chart_forex(path,symbol):
    df = get_symbol_df(path,symbol)
    fig = go.Figure()
    fig.add_trace(go.Candlestick(x=df['date'],
                    open=df['open'],
                    high=df['high'],
                    low=df['low'],
                    close=df['close']))
    fig.add_trace(go.Scatter(
        x=df.query('lots == 0.1')['date'],
        y=df.query('lots == 0.1')['close'],
        mode='markers',
        marker={"size":5,"color":"blue"},
        name='lot 0.1'
    ))
    fig.add_trace(go.Scatter(
        x=df.query('lots == 0')['date'],
        y=df.query('lots == 0')['close'],
        mode='markers',
        marker={"size":5,"color":"pink"},
        name='lot 0'
    ))
    fig.add_trace(go.Scatter(
        x=df.query('lots == -0.1')['date'],
        y=df.query('lots == -0.1')['close'],
        mode='markers',
        marker={"size":5,"color":"yellow"},
        name='lot -0.1'
    ))
    fig.update_layout(
        title=f'Chart for {symbol}',
        xaxis_title = "Date",
        yaxis_title="close",
    )
    fig.show()

def plot_chart_stocks(path,symbol):
    df = get_symbol_df(path,symbol)
    fig = go.Figure()
    fig.add_trace(go.Candlestick(x=df['date'],
                    open=df['open'],
                    high=df['high'],
                    low=df['low'],
                    close=df['close']))
    
    q = 'weights >= {} & weights <= {}'
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0,0.1))['date'],
        y=df.query(q.format(0,0.1))['close'],
        mode='markers',
        marker={"size":5,"color":(1,0,0)},
        name='weights 0.0-0.1'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.1,0.2))['date'],
        y=df.query(q.format(0.1,0.2))['close'],
        mode='markers',
        marker={"size":5,"color":(0,1,0)},
        name='weights 0.1-0.2'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.2,0.3))['date'],
        y=df.query(q.format(0.2,0.3))['close'],
        mode='markers',
        marker={"size":5,"color":'#2ff3e0'},
        name='weights 0.2-0.3'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.3,0.4))['date'],
        y=df.query(q.format(0.3,0.4))['close'],
        mode='markers',
        marker={"size":5,"color":"#f8d210"},
        name='weights 0.3-0.4'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.4,0.5))['date'],
        y=df.query(q.format(0.4,0.5))['close'],
        mode='markers',
        marker={"size":5,"color":"#fa26a0"},
        name='weights 0.4-0.5'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.5,0.6))['date'],
        y=df.query(q.format(0.5,0.6))['close'],
        mode='markers',
        marker={"size":5,"color":"#f51720"},
        name='weights 0.5-0.6'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.6,0.7))['date'],
        y=df.query(q.format(0.6,0.7))['close'],
        mode='markers',
        marker={"size":5,"color":"#050a30"},
        name='weights 0.6-0.7'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.7,0.8))['date'],
        y=df.query(q.format(0.7,0.8))['close'],
        mode='markers',
        marker={"size":5,"color":"#000c66"},
        name='weights 0.7-0.8'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.8,0.9))['date'],
        y=df.query(q.format(0.8,0.9))['close'],
        mode='markers',
        marker={"size":5,"color":"#0000ff"},
        name='weights 0.8-0.9'
    ))
    fig.add_trace(go.Scatter(
        x=df.query(q.format(0.9,1))['date'],
        y=df.query(q.format(0.9,1))['close'],
        mode='markers',
        marker={"size":5,"color":"#7ec8e3"},
        name='weights 0.9-1'
    ))
    fig.update_layout(
        title=f'Chart for {symbol}',
        xaxis_title = "Date",
        yaxis_title="close",
    )
    fig.show()


# import pyfolio
# from copy import deepcopy
# from pyfolio import timeseries
# from rltrade import config
# from rltrade.data import IBKRDownloader

# def get_daily_return(df,value_col_name="account_value"):
#     df = deepcopy(df)
#     df = df.groupby(['date'])[value_col_name].sum().pct_change(1).reset_index()
#     df.set_index('date',inplace=True,drop=True)
#     df.index = df.index.tz_localize("UTC")
#     return pd.Series(df[value_col_name],index=df.index)

# def convert_daily_return_to_pyfolio_ts(df):
#     strategy_ret = df.copy()
#     strategy_ret['date'] = pd.to_datetime(strategy_ret['date'])
#     strategy_ret.set_index('date',drop=False,inplace=True)
#     strategy_ret.index = strategy_ret.index.tz_localize("UTC")
#     del strategy_ret['date']
#     return pd.Series(strategy_ret['daily_return'].to_numpy(),index=strategy_ret.index)

# def get_baseline(ticker,sec_types,exchanges,start,end,start_time,end_time,mode,demo):
#     if mode == 'daily':
#         df = IBKRDownloader(start_date=start,
#         end_date=end,ticker_list=ticker,
#         sec_types=sec_types,
#         exchanges=exchanges,
#         start_time=start_time,
#         end_time=end_time,
#         demo=demo,
#         ).fetch_data()
#     elif mode == 'min':
#         df = IBKRDownloader(start_date=start,
#         end_date=end,ticker_list=ticker,
#         sec_types=sec_types,
#         exchanges=exchanges,
#         start_time=start_time,
#         end_time=end_time,
#         demo=demo,
#         ).fetch_min_data()
#     df = df.sort_values(by=['date','tic']).reset_index(drop=True)
#     return df

# def backtest_stats(df,
#                 baseline_start = config.START_TRADE_DATE,
#                 baseline_end = config.END_DATE,
#                 start_time="09:30:00",
#                 end_time = "15:59:00",
#                 baseline_ticker=[],
#                 sec_types=[],
#                 exchanges=[],
#                 value_col_name="account_value",
#                 mode='daily',
#                 demo=True):

#     if value_col_name == "daily_return":
#         df = convert_daily_return_to_pyfolio_ts(df)
    
#     if value_col_name == "account_value":
#         df = get_daily_return(df,value_col_name=value_col_name)
    
#     perf_stats_all = timeseries.perf_stats(returns=df,turnover_denom="AGB")
    
#     baseline_df = get_baseline(ticker=baseline_ticker,sec_types=sec_types,exchanges=exchanges,
#                                 start=baseline_start,end=baseline_end,
#                                 start_time=start_time,end_time=end_time,
#                                 mode=mode,demo=demo)
#     baseline_df = baseline_df[(baseline_df['date']>=baseline_start) & (baseline_df['date'] <= baseline_end)]
#     baseline_df['date'] = pd.to_datetime(baseline_df['date'],format="%Y-%m-%d")
#     baseline_df = baseline_df.fillna(method='ffill').fillna(method='bfill')
#     baseline_returns = get_daily_return(baseline_df,value_col_name="close")
#     perf_baseline = timeseries.perf_stats(returns=baseline_returns,turnover_denom='AGB')

#     df = pd.concat((perf_stats_all,perf_baseline),axis=1)
#     df.columns = ["model","baseline"]
#     return df


# def backtest_plot(account_value,
#                 baseline_start = config.START_TRADE_DATE,
#                 baseline_end = config.END_DATE,
#                 start_time="09:30:00",
#                 end_time="15:59:00",
#                 baseline_ticker='^DJI',
#                 sec_types=[],
#                 exchanges=[],
#                 value_col_name="account_value",
#                 mode='daily',
#                 demo=True):
#     df = deepcopy(account_value)
#     df['date'] = pd.to_datetime(df['date'])

#     if value_col_name == "daily_return":
#         test_returns = convert_daily_return_to_pyfolio_ts(df)
    
#     if value_col_name == "account_value":
#         test_returns = get_daily_return(df,value_col_name=value_col_name)

#     baseline_df = get_baseline(ticker=baseline_ticker,
#                                 sec_types=sec_types,exchanges=exchanges,
#                                 start=baseline_start,end=baseline_end,
#                                 start_time=start_time,end_time=end_time,
#                                 mode=mode,demo=demo)

#     baseline_df['date'] = pd.to_datetime(baseline_df['date'],format="%Y-%m-%d")
#     baseline_df = pd.merge(df['date'],baseline_df,how='left',on='date')
#     baseline_df = baseline_df.fillna(method='ffill').fillna(method='bfill')
#     baseline_returns = get_daily_return(baseline_df,value_col_name="close")

#     with pyfolio.plotting.plotting_context(font_scale=1.1):
#         pyfolio.create_full_tear_sheet(
#             returns=test_returns,
#             benchmark_rets=baseline_returns,
#             set_context=False)